const express = require('express');
const path = require('path');
const fs = require('fs');
const cors = require('cors');
const nodemailer = require('nodemailer');
const { app: electronApp } = require('electron');

function createIntegratedServer(serverPath) {
  const app = express();
  
  // Middleware
  app.use(cors({
    origin: true,
    credentials: false,
  }));
  
  app.use(express.json());
  app.use(express.urlencoded({ extended: true }));
  
  // Set up view engine
  app.set('view engine', 'ejs');
  app.set('views', path.join(serverPath, 'views'));
  app.use(express.static(path.join(serverPath, 'public')));
  
  // Settings management
  const settingsPath = path.join(serverPath, 'config', 'settings.json');
  
  function getSettings() {
    try {
      if (fs.existsSync(settingsPath)) {
        const settings = JSON.parse(fs.readFileSync(settingsPath, 'utf8'));
        console.log('Loaded settings:', settings);
        return settings;
      }
    } catch (error) {
      console.error('Error reading settings:', error);
    }
    
    // Return default settings if file doesn't exist or can't be read
    const defaultSettings = {
      "SENDER_HOST": "",
      "SENDER_PORT": 587,
      "SENDER_MAIL": "",
      "SENDER_NAME": "",
      "SENDER_USER": "",
      "SENDER_PASS": "",
      "EMAILS_PER_HOUR": 300
    };
    
    // Try to create the settings file with defaults
    try {
      const configDir = path.dirname(settingsPath);
      if (!fs.existsSync(configDir)) {
        fs.mkdirSync(configDir, { recursive: true });
      }
      fs.writeFileSync(settingsPath, JSON.stringify(defaultSettings, null, 2));
      console.log('Created default settings file');
    } catch (error) {
      console.error('Failed to create default settings file:', error);
    }
    
    return defaultSettings;
  }
  
  function saveSettings(settings) {
    try {
      const configDir = path.dirname(settingsPath);
      if (!fs.existsSync(configDir)) {
        fs.mkdirSync(configDir, { recursive: true });
      }
      fs.writeFileSync(settingsPath, JSON.stringify(settings, null, 2));
      console.log('Settings saved successfully:', settings);
      return true;
    } catch (error) {
      console.error('Error saving settings:', error);
      return false;
    }
  }
  
  // Routes
  app.get('/', (req, res) => {
    res.redirect('/mailing');
  });
  
  app.get('/mailing', (req, res) => {
    try {
      res.render('MailMain');
    } catch (error) {
      console.error('Error rendering MailMain:', error);
      res.status(500).send('Error loading application');
    }
  });
  
  app.get('/mailing/settings', (req, res) => {
    try {
      res.render('settings');
    } catch (error) {
      console.error('Error rendering settings:', error);
      res.status(500).send('Error loading settings');
    }
  });
  
  app.get('/mailing/api/settings', (req, res) => {
    try {
      console.log('GET /mailing/api/settings called');
      const settings = getSettings();
      
      if (!settings || Object.keys(settings).length === 0) {
        console.error('No settings found or settings object is empty');
        return res.status(404).json({
          success: false,
          error: "No settings found"
        });
      }
      
      console.log('Returning settings:', settings);
      res.json({
        success: true,
        settings: settings
      });
    } catch (error) {
      console.error('Error getting settings:', error.message, error.stack);
      res.status(500).json({ 
        success: false, 
        error: "Failed to get settings: " + error.message
      });
    }
  });
  
  app.post('/mailing/api/settings', (req, res) => {
    try {
      const newSettings = req.body;
      console.log('POST /mailing/api/settings called with:', newSettings);
      
      // Validate required fields
      const requiredFields = ['SENDER_HOST', 'SENDER_PORT', 'SENDER_MAIL', 'SENDER_USER', 'SENDER_PASS'];
      for (const field of requiredFields) {
        if (!newSettings[field]) {
          console.error(`Missing required field: ${field}`);
          return res.status(400).json({ 
            success: false, 
            error: `Missing required field: ${field}` 
          });
        }
      }
      
      const success = saveSettings(newSettings);
      if (success) {
        res.json({ success: true, message: 'Settings saved successfully' });
      } else {
        res.status(500).json({ success: false, error: 'Failed to save settings' });
      }
    } catch (error) {
      console.error('Error saving settings:', error);
      res.status(500).json({ 
        success: false, 
        error: "Failed to save settings: " + error.message 
      });
    }
  });
  
  app.post('/mailing/api/test-connection', async (req, res) => {
    try {
      console.log('POST /mailing/api/test-connection called with:', req.body);
      const { SENDER_HOST, SENDER_PORT, SENDER_USER, SENDER_PASS } = req.body;
      
      // Validate required fields
      if (!SENDER_HOST || !SENDER_PORT || !SENDER_USER || !SENDER_PASS) {
        return res.status(400).json({ 
          success: false, 
          error: 'Missing required connection parameters' 
        });
      }
      
      const transporter = nodemailer.createTransport({
        host: SENDER_HOST,
        port: parseInt(SENDER_PORT),
        secure: false,
        auth: {
          user: SENDER_USER,
          pass: SENDER_PASS
        },
        tls: {
          rejectUnauthorized: false,
        },
      });
      
      await transporter.verify();
      console.log('Connection test successful');
      res.json({ success: true, message: 'Connection successful' });
    } catch (error) {
      console.error('SMTP test failed:', error);
      res.status(500).json({ success: false, error: error.message || 'Connection failed' });
    }
  });
  
  app.post('/mailing/send-email', async (req, res) => {
    try {
      const { emails, subject, content } = req.body;
      const settings = getSettings();
      
      if (!settings.SENDER_HOST || !settings.SENDER_USER) {
        return res.status(400).json({ 
          success: false, 
          error: 'SMTP settings not configured' 
        });
      }
      
      const transporter = nodemailer.createTransport({
        host: settings.SENDER_HOST,
        port: parseInt(settings.SENDER_PORT),
        secure: false,
        auth: {
          user: settings.SENDER_USER,
          pass: settings.SENDER_PASS
        },
        tls: {
          rejectUnauthorized: false,
        },
      });
      
      // Process emails into an array
      const emailList = Array.isArray(emails)
        ? emails
        : typeof emails === "string"
          ? emails.split(",").map((e) => e.trim())
          : [];

      // Filter valid emails
      const validEmails = emailList.filter((email) =>
        /^[^\s@]+@[^\s@]+\.[^\s@]+$/.test(email),
      );

      if (validEmails.length === 0) {
        return res.status(400).json({ error: "No valid email addresses provided" });
      }

      const results = [];
      let sentCount = 0;
      
      for (const [index, email] of validEmails.entries()) {
        try {
          const info = await transporter.sendMail({
            from: `"${settings.SENDER_NAME}" <${settings.SENDER_MAIL}>`,
            to: email,
            subject: subject,
            html: content
          });
          
          console.log(`Sent to ${email} (${index + 1}/${validEmails.length})`);
          results.push({ email, status: "sent", messageId: info.messageId });
          sentCount++;
        } catch (error) {
          console.error(`Failed to send to ${email}:`, error);
          results.push({ email, status: "failed", error: error.message });
        }
      }
      
      res.json({
        success: true,
        total: validEmails.length,
        sent: sentCount,
        failed: validEmails.length - sentCount,
        results: results
      });
      
    } catch (error) {
      console.error('Email sending error:', error);
      res.status(500).json({ success: false, error: error.message });
    }
  });
  
  // 404 handler
  app.use((req, res) => {
    try {
      res.status(404).render('404');
    } catch (error) {
      res.status(404).send('Page not found');
    }
  });
  
  return app;
}

module.exports = createIntegratedServer;