const { createLicenseManager } = require('electron-gumroad-license');
const { dialog, BrowserWindow, app } = require('electron');
const path = require('path');
const fs = require('fs');
const os = require('os');

class MaSenderLicenseManager {
  constructor() {
    this.licenseManager = null;
    this.productId = null;
    this.isLicensed = false;
    this.licenseWindow = null;
    this.licenseFile = path.join(os.homedir(), '.masender', 'license.json');
  }

  // Initialize with your Gumroad product ID
  initialize(productId) {
    console.log('🔧 License Manager: Initializing with Product ID:', productId);
    this.productId = productId;
    this.licenseManager = createLicenseManager(productId, {
      maxUses: 3, // Allow license to be used on 3 computers max
      disableEncryption: false // Keep encryption enabled for security
    });
    console.log('✅ License Manager: createLicenseManager completed');
    
    // Ensure license directory exists
    const licenseDir = path.dirname(this.licenseFile);
    if (!fs.existsSync(licenseDir)) {
      fs.mkdirSync(licenseDir, { recursive: true });
    }
    console.log('✅ License Manager: Initialization complete');
  }

  // Save license to local storage
  saveLicenseToFile(licenseKey, licenseData) {
    try {
      const data = {
        key: licenseKey,
        data: licenseData,
        timestamp: Date.now(),
        productId: this.productId
      };
      fs.writeFileSync(this.licenseFile, JSON.stringify(data, null, 2));
      console.log('License saved to local file');
      return true;
    } catch (error) {
      console.error('Failed to save license to file:', error);
      return false;
    }
  }

  // Load license from local storage
  loadLicenseFromFile() {
    try {
      if (!fs.existsSync(this.licenseFile)) {
        return null;
      }
      
      const data = JSON.parse(fs.readFileSync(this.licenseFile, 'utf8'));
      
      // Verify it's for the same product
      if (data.productId !== this.productId) {
        return null;
      }
      
      console.log('License loaded from local file');
      return data;
    } catch (error) {
      console.error('Failed to load license from file:', error);
      return null;
    }
  }

  // Check if user has a valid license
  async checkLicense() {
    if (!this.licenseManager) {
      console.error('License manager not initialized');
      return false;
    }

    try {
      // First try the Gumroad license manager
      const result = await this.licenseManager.checkCurrentLicense();
      this.isLicensed = result.valid;
      console.log('License check result:', result);
      
      if (result.valid) {
        return true;
      }
      
      // If Gumroad check fails, try local file
      const localLicense = this.loadLicenseFromFile();
      if (localLicense && localLicense.key) {
        console.log('Attempting to validate stored license key...');
        const validateResult = await this.licenseManager.addLicense(localLicense.key);
        if (validateResult.valid) {
          this.isLicensed = true;
          console.log('Stored license key is valid');
          return true;
        } else {
          console.log('Stored license key is no longer valid');
          // Remove invalid license file
          try {
            fs.unlinkSync(this.licenseFile);
          } catch (e) {}
        }
      }
      
      return false;
    } catch (error) {
      console.error('License check failed:', error);
      this.isLicensed = false;
      return false;
    }
  }

  // Validate and add a new license key
  async addLicense(licenseKey) {
    console.log('🔑 License Manager: addLicense called with key:', licenseKey.substring(0, 8) + '...');
    console.log('🔑 License Manager: Current Product ID:', this.productId);
    
    if (!this.licenseManager) {
      console.error('❌ License Manager: Not initialized');
      throw new Error('License manager not initialized');
    }

    try {
      console.log('🔑 License Manager: Calling licenseManager.addLicense...');
      const result = await this.licenseManager.addLicense(licenseKey);
      console.log('🔑 License Manager: Raw result:', JSON.stringify(result, null, 2));
      
      if (result.valid) {
        this.isLicensed = true;
        console.log('✅ License Manager: License added successfully');
        
        // Save license to local file for persistence
        this.saveLicenseToFile(licenseKey, result);
      } else {
        console.log('❌ License Manager: License validation failed');
      }
      return result;
    } catch (error) {
      console.error('❌ License Manager: addLicense error:', error.message);
      console.error('❌ License Manager: Error stack:', error.stack);
      throw error;
    }
  }

  // Show license entry dialog
  showLicenseDialog() {
    return new Promise((resolve) => {
      if (this.licenseWindow) {
        this.licenseWindow.focus();
        return;
      }

      this.licenseWindow = new BrowserWindow({
        width: 500,
        height: 600,
        resizable: false,
        modal: true,
        show: false,
        autoHideMenuBar: true,
        webPreferences: {
          nodeIntegration: true,
          contextIsolation: false,
          devTools: false,
          webSecurity: true,
          allowRunningInsecureContent: false,
          experimentalFeatures: false
        },
        icon: path.join(__dirname, 'build', 'icon.png'),
        title: 'MaSender - License Required'
      });

      this.licenseWindow.loadFile(path.join(__dirname, 'license-dialog.html'));

      this.licenseWindow.once('ready-to-show', () => {
        this.licenseWindow.show();
      });

      this.licenseWindow.on('closed', () => {
        this.licenseWindow = null;
        resolve(this.isLicensed);
      });

      // Additional security for license window
      this.licenseWindow.webContents.on('before-input-event', (event, input) => {
        // Prevent dev tools shortcuts
        if (
          input.key === 'F12' ||
          (input.key === 'I' && input.control && input.shift) ||
          (input.key === 'i' && input.control && input.shift) ||
          (input.key === 'J' && input.control && input.shift) ||
          (input.key === 'j' && input.control && input.shift) ||
          (input.key === 'U' && input.control) ||
          (input.key === 'u' && input.control)
        ) {
          event.preventDefault();
        }
      });

      // Disable right-click context menu
      this.licenseWindow.webContents.on('context-menu', (event) => {
        event.preventDefault();
      });

      // Prevent new windows
      this.licenseWindow.webContents.on('new-window', (event) => {
        event.preventDefault();
      });
    });
  }

  // Check if app should start (requires valid license in production)
  async shouldAllowAppStart() {
    // If no product ID set, allow app to start (development mode)
    if (!this.productId) {
      console.log('No product ID set - development mode');
      return true;
    }

    const hasValidLicense = await this.checkLicense();
    
    if (!hasValidLicense) {
      // Show license dialog and wait for result
      const licenseResult = await this.showLicenseDialog();
      return licenseResult;
    }

    return true;
  }

  // Get license status information
  async getLicenseInfo() {
    if (!this.licenseManager) {
      return { valid: false, info: null };
    }

    try {
      const result = await this.licenseManager.checkCurrentLicense();
      return {
        valid: result.valid,
        info: result.license || null
      };
    } catch (error) {
      return { valid: false, info: null };
    }
  }

  // Clear stored license (for testing or license reset)
  clearStoredLicense() {
    try {
      if (fs.existsSync(this.licenseFile)) {
        fs.unlinkSync(this.licenseFile);
        console.log('Stored license cleared');
      }
      this.isLicensed = false;
      return true;
    } catch (error) {
      console.error('Failed to clear stored license:', error);
      return false;
    }
  }
}

module.exports = new MaSenderLicenseManager();