const express = require("express");
const path = require("path");
const morgan = require("morgan");
const cors = require("cors");
const fs = require("fs");

// Make sure config directory exists
const configDir = path.join(__dirname, "config");
if (!fs.existsSync(configDir)) {
  fs.mkdirSync(configDir);
}

// Use hardcoded server configuration instead of config.json
const config = {
  server: {
    port: 3000,
    frontendUrl: "http://localhost"
  }
};
console.log("Using built-in server configuration");
const app = express();

// Middleware
app.use(morgan("dev"));

// Add request logging middleware
app.use((req, res, next) => {
  console.log(`[${new Date().toISOString()}] ${req.method} ${req.url}`);
  console.log('Content-Type:', req.headers['content-type']);
  next();
});

// CORS configuration (must be before other middleware)
app.use(
  cors({
    origin: true, // Allow all origins in development
    credentials: false,
  }),
);

// Body parsing middleware
app.use(express.urlencoded({ extended: true, limit: '10mb' }));
app.use(express.json({ limit: '10mb' }));

app.set("view engine", "ejs");
app.set("trust proxy", 1);

// View engine setup
app.set("views", path.join(__dirname, "views"));
app.use(express.static(path.join(__dirname, "public")));

// Routes
const indexRouter = require("./routers/indexRouter");

// Use only the indexRouter which includes mailingRouter
app.use("/", indexRouter);

// Add catch-all route for debugging
app.use('*', (req, res, next) => {
  console.log(`ROUTE NOT FOUND: ${req.method} ${req.originalUrl}`);
  console.log('Available routes should include: /mailing/api/import-emails');
  next();
});

// Server start
const startServer = (port) => {
  return new Promise((resolve, reject) => {
    const server = app.listen(port)
      .on('listening', () => {
        console.log(`Server is running on http://localhost:${port}`);
        resolve({ server, port });
      })
      .on('error', (err) => {
        if (err.code === 'EADDRINUSE') {
          console.log(`Port ${port} is busy, trying ${port + 1}...`);
          server.close();
          resolve(startServer(port + 1));
        } else {
          reject(err);
        }
      });
  });
};

// Try ports starting from default port
const PORT = 3000;
startServer(PORT).then(({ port }) => {
  // Export the port so the Electron app can find it
  global.serverPort = port;
  console.log(`Server is using built-in server settings and settings.json for email settings`);
});
