const nodemailer = require("nodemailer");
const fs = require("fs");
const path = require("path");

// Settings path
const settingsPath = path.join(__dirname, "../config/settings.json");

// Make sure the config directory exists
if (!fs.existsSync(path.join(__dirname, "../config"))) {
  fs.mkdirSync(path.join(__dirname, "../config"));
}

// Create default settings.json if it doesn't exist
if (!fs.existsSync(settingsPath)) {
  const defaultSettings = {
    SENDER_HOST: "",
    SENDER_PORT: 587,
    SENDER_MAIL: "",
    SENDER_NAME: "",
    SENDER_USER: "",
    SENDER_PASS: "",
    EMAILS_PER_HOUR: 300,
  };
  fs.writeFileSync(settingsPath, JSON.stringify(defaultSettings, null, 2));
  // console.log('Created default settings.json file - Please configure your SMTP settings');
}

// Load settings from settings.json
const loadSettings = () => {
  console.log("Loading settings...");
  try {
    // Load from settings.json (user settings)
    if (fs.existsSync(settingsPath)) {
      // console.log('settings.json exists, loading from:', settingsPath);
      try {
        const fileContents = fs.readFileSync(settingsPath, "utf8");
        const settings = JSON.parse(fileContents);
        // console.log('Successfully loaded settings from settings.json');
        return settings;
      } catch (parseError) {
        console.error(
          "Error parsing settings.json, file content:",
          fs.readFileSync(settingsPath, "utf8"),
        );
        console.error("Parse error:", parseError);
        // If settings.json is invalid, recreate it with default values
        const defaultSettings = {
          SENDER_HOST: "",
          SENDER_PORT: 587,
          SENDER_MAIL: "",
          SENDER_NAME: "",
          SENDER_USER: "",
          SENDER_PASS: "",
          EMAILS_PER_HOUR: 300,
        };
        fs.writeFileSync(
          settingsPath,
          JSON.stringify(defaultSettings, null, 2),
        );
        // console.log('Recreated default settings.json file after parse error');
        return defaultSettings;
      }
    }
  } catch (error) {
    console.error("Error loading settings from settings.json:", error);
  }

  // Default values if settings.json not found or error occurred
  console.warn("No configuration found, creating default settings");
  const defaultSettings = {
    SENDER_HOST: "",
    SENDER_PORT: 587,
    SENDER_MAIL: "",
    SENDER_NAME: "",
    SENDER_USER: "",
    SENDER_PASS: "",
    EMAILS_PER_HOUR: 300,
  };

  // Try to create settings.json with default values
  try {
    fs.writeFileSync(settingsPath, JSON.stringify(defaultSettings, null, 2));
    // console.log('Created settings.json with default values');
  } catch (error) {
    console.error("Failed to create settings.json:", error);
  }

  return defaultSettings;
};

// Get current settings (call function to initialize)
console.log("Initializing settings...");
let settings = loadSettings();
// console.log('Settings initialized:', settings);

// Nodemailer transporter
const createTransporter = () => {
  return nodemailer.createTransport({
    host: settings.SENDER_HOST,
    port: settings.SENDER_PORT,
    secure: false,
    auth: {
      user: settings.SENDER_USER,
      pass: settings.SENDER_PASS,
    },
    tls: {
      rejectUnauthorized: false,
    },
  });
};

let transporter = createTransporter();

// Email sending rate limiter (configurable based on user settings)
// Maximum allowed: 1000 emails per hour
const MAX_EMAILS_PER_HOUR = 1000;
const MIN_EMAILS_PER_HOUR = 1;
// For small batches (less than 10), we'll send without delays
const SMALL_BATCH_THRESHOLD = 10;
let lastSentTime = 0;

// Function to calculate delay between emails based on rate limit
const calculateDelayMs = (emailsPerHour) => {
  // Validate and clamp the rate
  const clampedRate = Math.max(
    MIN_EMAILS_PER_HOUR,
    Math.min(MAX_EMAILS_PER_HOUR, emailsPerHour),
  );
  // Convert emails per hour to milliseconds between emails
  return Math.floor(3600000 / clampedRate); // 3600000 ms = 1 hour
};

exports.getHomePage = (req, res) => {
  res.render("MailMain");
};

exports.getSettingsPage = (req, res) => {
  res.render("settings");
};

exports.sendEmail = async (req, res) => {
  try {
    // Reload settings before sending emails
    const currentSettings = loadSettings();

    // Check if SMTP settings are configured
    if (
      !currentSettings.SENDER_HOST ||
      !currentSettings.SENDER_MAIL ||
      !currentSettings.SENDER_USER ||
      !currentSettings.SENDER_PASS
    ) {
      return res.status(400).json({
        error:
          "SMTP settings not configured. Please configure your email settings first.",
        needsConfiguration: true,
      });
    }

    transporter = createTransporter();

    const { emails, subject, content } = req.body;

    // Validate required fields
    if (!emails || !subject || !content) {
      return res.status(400).json({ error: "Missing required fields" });
    }

    // Process emails into an array
    const emailList = Array.isArray(emails)
      ? emails
      : typeof emails === "string"
        ? emails.split(",").map((e) => e.trim())
        : [];

    // Filter valid emails
    const validEmails = emailList.filter((email) =>
      /^[^\s@]+@[^\s@]+\.[^\s@]+$/.test(email),
    );

    if (validEmails.length === 0) {
      return res
        .status(400)
        .json({ error: "No valid email addresses provided" });
    }

    // Track sending progress
    const results = [];
    let sentCount = 0;
    const startTime = Date.now();

    // Get rate limit from settings
    const emailsPerHour = currentSettings.EMAILS_PER_HOUR || 300;
    const clampedRate = Math.max(
      MIN_EMAILS_PER_HOUR,
      Math.min(MAX_EMAILS_PER_HOUR, emailsPerHour),
    );

    // Determine if we need rate limiting based on batch size
    const needsRateLimit = validEmails.length >= SMALL_BATCH_THRESHOLD;
    const delayMs = needsRateLimit ? calculateDelayMs(clampedRate) : 0;

    // Send emails with adaptive rate limiting
    for (const [index, email] of validEmails.entries()) {
      try {
        // Calculate delay needed to stay within limits (only if batch is large enough)
        if (needsRateLimit) {
          const elapsed = Date.now() - startTime;
          const targetTime = index * delayMs;
          const delay = Math.max(0, targetTime - elapsed);

          if (delay > 0) {
            console.log(`Waiting ${delay}ms before next send...`);
            await new Promise((resolve) => setTimeout(resolve, delay));
          }
        }

        const info = await transporter.sendMail({
          from: `"${currentSettings.SENDER_NAME}" <${currentSettings.SENDER_MAIL}>`,
          to: email,
          subject: subject,
          html: content,
        });

        console.log(
          `[${new Date().toISOString()}] Sent to ${email} (${index + 1}/${validEmails.length})`,
        );
        results.push({ email, status: "sent", messageId: info.messageId });
        sentCount++;
        lastSentTime = Date.now();
      } catch (error) {
        console.error(`Failed to send to ${email}:`, error.message);
        results.push({ email, status: "failed", error: error.message });

        // If we get SMTP errors, pause for 5 minutes
        if (error.responseCode >= 400 && error.responseCode < 500) {
          console.error("SMTP error detected, pausing for 5 minutes...");
          await new Promise((resolve) => setTimeout(resolve, 300000));
        }
      }
    }

    res.json({
      success: true,
      total: validEmails.length,
      sent: sentCount,
      failed: validEmails.length - sentCount,
      results,
      estimatedDuration:
        validEmails.length < SMALL_BATCH_THRESHOLD
          ? "less than a minute"
          : `${((validEmails.length * delayMs) / 60000).toFixed(1)} minutes`,
      rateLimit:
        validEmails.length < SMALL_BATCH_THRESHOLD
          ? "No rate limit for small batches (less than 10 emails)"
          : `${clampedRate} emails/hour (1 every ${(delayMs / 1000).toFixed(1)} seconds)`,
      configuredRate: clampedRate,
    });
  } catch (error) {
    console.error("Email sending error:", error);
    res.status(500).json({ error: "Internal server error" });
  }
};

// Get current email settings
exports.getSettings = (req, res) => {
  try {
    // console.log('GET /api/settings called');

    // Force reload settings to get the latest configuration
    const currentSettings = loadSettings();

    // Check if settings object is valid (not null/undefined and has at least one property)
    if (!currentSettings || Object.keys(currentSettings).length === 0) {
      console.error("No settings found or settings object is empty");
      return res.status(404).json({
        success: false,
        error: "No settings found",
      });
    }

    // Log what we're returning for debugging
    // console.log('Returning settings:', currentSettings);

    // Return success response with settings
    return res.json({
      success: true,
      settings: currentSettings,
    });
  } catch (error) {
    console.error("Error getting settings:", error.message, error.stack);
    return res.status(500).json({
      success: false,
      error: "Failed to get settings: " + error.message,
    });
  }
};

// Save email settings
exports.saveSettings = (req, res) => {
  try {
    const newSettings = req.body;

    // Validate required fields
    const requiredFields = [
      "SENDER_HOST",
      "SENDER_PORT",
      "SENDER_MAIL",
      "SENDER_USER",
      "SENDER_PASS",
    ];
    for (const field of requiredFields) {
      if (!newSettings[field] || newSettings[field].toString().trim() === "") {
        return res.status(400).json({
          success: false,
          error: `Missing required field: ${field}`,
        });
      }
    }

    // Validate and clamp EMAILS_PER_HOUR if provided
    if (newSettings.EMAILS_PER_HOUR !== undefined) {
      const emailsPerHour = parseInt(newSettings.EMAILS_PER_HOUR);
      if (isNaN(emailsPerHour) || emailsPerHour < MIN_EMAILS_PER_HOUR) {
        return res.status(400).json({
          success: false,
          error: `Emails per hour must be at least ${MIN_EMAILS_PER_HOUR}`,
        });
      }
      if (emailsPerHour > MAX_EMAILS_PER_HOUR) {
        return res.status(400).json({
          success: false,
          error: `Emails per hour cannot exceed ${MAX_EMAILS_PER_HOUR} for system stability`,
        });
      }
      newSettings.EMAILS_PER_HOUR = emailsPerHour;
    } else {
      // Set default if not provided
      newSettings.EMAILS_PER_HOUR = 300;
    }

    // console.log('Saving new settings:', newSettings);

    // Make sure the config directory exists
    const configDir = path.join(__dirname, "../config");
    if (!fs.existsSync(configDir)) {
      fs.mkdirSync(configDir);
    }

    // Save to file
    fs.writeFileSync(settingsPath, JSON.stringify(newSettings, null, 2));
    // console.log('Settings saved to:', settingsPath);

    // Update transporter
    Object.assign(settings, newSettings);
    transporter = createTransporter();

    res.json({
      success: true,
      message: "Settings saved successfully",
    });
  } catch (error) {
    console.error("Error saving settings:", error);
    res.status(500).json({
      success: false,
      error: "Failed to save settings: " + error.message,
    });
  }
};

// Test email connection
exports.testConnection = async (req, res) => {
  try {
    const testSettings = req.body;

    // Create a test transporter
    const testTransporter = nodemailer.createTransport({
      host: testSettings.SENDER_HOST,
      port: testSettings.SENDER_PORT,
      secure: false,
      auth: {
        user: testSettings.SENDER_USER,
        pass: testSettings.SENDER_PASS,
      },
      tls: {
        rejectUnauthorized: false,
      },
    });

    // Verify connection
    await testTransporter.verify();

    res.json({
      success: true,
      message: "Connection successful",
    });
  } catch (error) {
    console.error("Connection test failed:", error);
    res.status(500).json({
      success: false,
      error: error.message || "Connection failed",
    });
  }
};
