// Connection Monitor Utility for MaSender
// Provides consistent internet connection monitoring across the application

class ConnectionMonitor {
  constructor() {
    this.isElectronApp = this.detectElectronEnvironment();
    this.isConnected = true;
    this.lastConnectionStatus = true;
    this.listeners = [];
    this.checkInterval = null;
    this.initialized = false;
    
    console.log(`ConnectionMonitor: Running in ${this.isElectronApp ? 'Electron' : 'Browser'} mode`);
  }

  detectElectronEnvironment() {
    try {
      return !!(window.electron && window.electron.connection);
    } catch (e) {
      return false;
    }
  }

  init() {
    if (this.initialized) return;
    
    if (this.isElectronApp) {
      this.initElectronMonitoring();
    } else {
      this.initBrowserMonitoring();
    }
    
    this.initialized = true;
    console.log('ConnectionMonitor: Initialized');
  }

  initElectronMonitoring() {
    // Listen for connection changes from main process
    window.electron.ipcRenderer.on('connection-changed', (isConnected) => {
      this.handleConnectionChange(isConnected);
    });

    // Initial connection check
    setTimeout(() => {
      const initialStatus = window.electron.connection.check();
      this.handleConnectionChange(initialStatus);
    }, 1000);
  }

  initBrowserMonitoring() {
    // Check initial status
    this.handleConnectionChange(navigator.onLine);
    
    // Listen for browser online/offline events
    window.addEventListener('online', () => this.handleConnectionChange(true));
    window.addEventListener('offline', () => this.handleConnectionChange(false));
    
    // Periodic check for better accuracy
    this.checkInterval = setInterval(() => {
      this.performConnectionCheck();
    }, 30000); // Check every 30 seconds
  }

  async performConnectionCheck() {
    try {
      const response = await fetch('https://www.google.com/favicon.ico', {
        method: 'HEAD',
        mode: 'no-cors',
        cache: 'no-cache'
      });
      this.handleConnectionChange(true);
    } catch (error) {
      this.handleConnectionChange(false);
    }
  }

  handleConnectionChange(isConnected) {
    if (isConnected === this.lastConnectionStatus) return;
    
    this.lastConnectionStatus = isConnected;
    this.isConnected = isConnected;
    
    // Notify all listeners
    this.listeners.forEach(callback => {
      try {
        callback(isConnected);
      } catch (error) {
        console.error('ConnectionMonitor: Error in listener callback:', error);
      }
    });
    
    // Show appropriate toast notification
    this.showConnectionToast(isConnected);
    
    console.log(`ConnectionMonitor: Connection status changed to ${isConnected ? 'online' : 'offline'}`);
  }

  showConnectionToast(isConnected) {
    if (isConnected) {
      this.showToast('✅ Internet connection restored! Email services are now available.', 'success', 4000);
    } else {
      this.showToast('⚠️ No internet connection detected. The app will continue to work, but email features may be limited.', 'warning', 6000);
    }
  }

  showToast(message, type = 'info', duration = 4000) {
    // Use Electron toast if available
    if (this.isElectronApp && window.electron && window.electron.toast) {
      window.electron.toast.show(message, type, duration);
      return;
    }
    
    // Fallback to browser toast
    this.showBrowserToast(message, type, duration);
  }

  showBrowserToast(message, type, duration) {
    const toast = document.createElement('div');
    toast.className = `connection-toast toast-${type}`;
    toast.innerHTML = `
      <div class="toast-content">
        <span class="toast-icon">${this.getToastIcon(type)}</span>
        <span class="toast-message">${message}</span>
        <button class="toast-close" onclick="this.parentElement.parentElement.remove()">×</button>
      </div>
    `;
    
    // Add styles if not already present
    if (!document.getElementById('connection-toast-styles')) {
      const styles = document.createElement('style');
      styles.id = 'connection-toast-styles';
      styles.textContent = `
        .connection-toast {
          position: fixed;
          top: 20px;
          right: 20px;
          min-width: 320px;
          max-width: 450px;
          background: white;
          border-radius: 10px;
          box-shadow: 0 6px 20px rgba(0, 0, 0, 0.15);
          z-index: 10000;
          transform: translateX(120%);
          opacity: 0;
          transition: all 0.4s cubic-bezier(0.175, 0.885, 0.32, 1.275);
          border-left: 4px solid #007bff;
          font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif;
          backdrop-filter: blur(10px);
        }
        .connection-toast.show {
          transform: translateX(0);
          opacity: 1;
        }
        .connection-toast .toast-content {
          display: flex;
          align-items: center;
          padding: 14px 18px;
          gap: 10px;
        }
        .connection-toast .toast-icon {
          font-size: 20px;
          flex-shrink: 0;
          display: flex;
          align-items: center;
          justify-content: center;
          width: 24px;
          height: 24px;
        }
        .connection-toast .toast-message {
          flex: 1;
          font-size: 14px;
          color: #333;
          line-height: 1.5;
          font-weight: 500;
        }
        .connection-toast .toast-close {
          background: none;
          border: none;
          font-size: 20px;
          cursor: pointer;
          color: #999;
          padding: 0;
          width: 24px;
          height: 24px;
          display: flex;
          align-items: center;
          justify-content: center;
          border-radius: 50%;
          transition: all 0.2s;
          font-weight: bold;
        }
        .connection-toast .toast-close:hover {
          background: #f5f5f5;
          color: #666;
        }
        .connection-toast.toast-success {
          border-left-color: #28a745;
          background: linear-gradient(135deg, #ffffff 0%, #f8fff9 100%);
        }
        .connection-toast.toast-warning {
          border-left-color: #ffc107;
          background: linear-gradient(135deg, #ffffff 0%, #fffbf0 100%);
        }
        .connection-toast.toast-error {
          border-left-color: #dc3545;
          background: linear-gradient(135deg, #ffffff 0%, #fff5f5 100%);
        }
        .connection-toast.toast-info {
          border-left-color: #007bff;
          background: linear-gradient(135deg, #ffffff 0%, #f0f8ff 100%);
        }
      `;
      document.head.appendChild(styles);
    }
    
    document.body.appendChild(toast);
    
    // Animate in
    setTimeout(() => toast.classList.add('show'), 100);
    
    // Auto remove
    if (duration > 0) {
      setTimeout(() => {
        toast.classList.remove('show');
        setTimeout(() => {
          if (toast.parentNode) {
            toast.remove();
          }
        }, 400);
      }, duration);
    }
  }

  getToastIcon(type) {
    switch (type) {
      case 'success': return '✓';
      case 'warning': return '⚠';
      case 'error': return '✕';
      case 'info':
      default: return 'ℹ';
    }
  }

  // Public API methods
  getCurrentStatus() {
    return this.isConnected;
  }

  addListener(callback) {
    if (typeof callback === 'function') {
      this.listeners.push(callback);
      return () => this.removeListener(callback);
    }
  }

  removeListener(callback) {
    const index = this.listeners.indexOf(callback);
    if (index > -1) {
      this.listeners.splice(index, 1);
    }
  }

  destroy() {
    if (this.checkInterval) {
      clearInterval(this.checkInterval);
    }
    
    if (!this.isElectronApp) {
      window.removeEventListener('online', this.handleConnectionChange);
      window.removeEventListener('offline', this.handleConnectionChange);
    }
    
    this.listeners = [];
    this.initialized = false;
    
    console.log('ConnectionMonitor: Destroyed');
  }

  // Utility method to check if network operations should be attempted
  shouldAttemptNetworkOperation() {
    return this.isConnected;
  }

  // Method to show custom connection warning for specific operations
  showNetworkRequiredWarning(operationName = 'this operation') {
    if (!this.isConnected) {
      this.showToast(
        `🌐 Internet connection required for ${operationName}. Please check your connection and try again.`,
        'warning',
        5000
      );
      return false;
    }
    return true;
  }
}

// Create and export global instance
window.connectionMonitor = new ConnectionMonitor();

// Auto-initialize when DOM is ready
if (document.readyState === 'loading') {
  document.addEventListener('DOMContentLoaded', () => {
    window.connectionMonitor.init();
  });
} else {
  // DOM is already ready
  window.connectionMonitor.init();
}

// Export for module systems
if (typeof module !== 'undefined' && module.exports) {
  module.exports = ConnectionMonitor;
}