// Connection monitoring and toast notifications
let isElectronApp = false;
let lastConnectionStatus = true;

// Check if running in Electron
try {
  if (window.electron) {
    isElectronApp = true;
    console.log('Running in Electron app');
  }
} catch (e) {
  console.log('Running in browser');
}

// Initialize connection monitoring when page loads
document.addEventListener('DOMContentLoaded', () => {
  if (isElectronApp) {
    initializeConnectionMonitoring();
  } else {
    // For browser fallback
    initializeBrowserConnectionMonitoring();
  }
});

// Electron connection monitoring
function initializeConnectionMonitoring() {
  if (!isElectronApp) return;
  
  // Listen for connection changes from main process
  window.electron.ipcRenderer.on('connection-changed', (isConnected) => {
    handleConnectionChange(isConnected);
  });
  
  // Initial connection check
  setTimeout(() => {
    const initialStatus = window.electron.connection.check();
    handleConnectionChange(initialStatus);
  }, 1000);
}

// Browser fallback connection monitoring
function initializeBrowserConnectionMonitoring() {
  // Check initial status
  handleConnectionChange(navigator.onLine);
  
  // Listen for online/offline events
  window.addEventListener('online', () => handleConnectionChange(true));
  window.addEventListener('offline', () => handleConnectionChange(false));
}

// Handle connection status changes
function handleConnectionChange(isConnected) {
  if (isConnected === lastConnectionStatus) return;
  
  lastConnectionStatus = isConnected;
  
  if (isConnected) {
    showToast('✅ Internet connection restored! You can now send emails.', 'success', 4000);
  } else {
    showToast('⚠️ No internet connection detected. The app will continue to work, but email sending may be limited.', 'warning', 6000);
  }
}

// Toast notification system
function showToast(message, type = 'info', duration = 4000) {
  // Use Electron toast if available
  if (isElectronApp && window.electron && window.electron.toast) {
    window.electron.toast.show(message, type, duration);
    return;
  }
  
  // Fallback for browser
  showBrowserToast(message, type, duration);
}

// Browser fallback toast
function showBrowserToast(message, type, duration) {
  const toast = document.createElement('div');
  toast.className = `browser-toast toast-${type}`;
  toast.innerHTML = `
    <div class="toast-content">
      <span class="toast-icon">${getToastIcon(type)}</span>
      <span class="toast-message">${message}</span>
      <button class="toast-close" onclick="this.parentElement.parentElement.remove()">×</button>
    </div>
  `;
  
  // Add styles if not already present
  if (!document.getElementById('browser-toast-styles')) {
    const styles = document.createElement('style');
    styles.id = 'browser-toast-styles';
    styles.textContent = `
      .browser-toast {
        position: fixed;
        top: 20px;
        right: 20px;
        min-width: 300px;
        max-width: 400px;
        background: white;
        border-radius: 8px;
        box-shadow: 0 4px 12px rgba(0, 0, 0, 0.15);
        z-index: 10000;
        transform: translateX(100%);
        opacity: 0;
        transition: all 0.3s ease;
        border-left: 4px solid #007bff;
        font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif;
      }
      .browser-toast.show {
        transform: translateX(0);
        opacity: 1;
      }
      .browser-toast .toast-content {
        display: flex;
        align-items: center;
        padding: 12px 16px;
        gap: 8px;
      }
      .browser-toast .toast-icon {
        font-size: 18px;
        flex-shrink: 0;
      }
      .browser-toast .toast-message {
        flex: 1;
        font-size: 14px;
        color: #333;
        line-height: 1.4;
      }
      .browser-toast .toast-close {
        background: none;
        border: none;
        font-size: 18px;
        cursor: pointer;
        color: #666;
        padding: 0;
        width: 20px;
        height: 20px;
        display: flex;
        align-items: center;
        justify-content: center;
        border-radius: 50%;
        transition: background 0.2s;
      }
      .browser-toast .toast-close:hover {
        background: #f0f0f0;
      }
      .browser-toast.toast-success {
        border-left-color: #28a745;
      }
      .browser-toast.toast-warning {
        border-left-color: #ffc107;
      }
      .browser-toast.toast-error {
        border-left-color: #dc3545;
      }
      .browser-toast.toast-info {
        border-left-color: #007bff;
      }
    `;
    document.head.appendChild(styles);
  }
  
  document.body.appendChild(toast);
  
  setTimeout(() => toast.classList.add('show'), 10);
  
  if (duration > 0) {
    setTimeout(() => {
      toast.classList.remove('show');
      setTimeout(() => toast.remove(), 300);
    }, duration);
  }
}

function getToastIcon(type) {
  switch (type) {
    case 'success': return '✓';
    case 'warning': return '⚠';
    case 'error': return '✕';
    case 'info':
    default: return 'ℹ';
  }
}

// Enhanced send button with connection check
document.getElementById("sendBtn").addEventListener("click", async () => {
  const emails = document.getElementById("emails").value;
  const subject = document.getElementById("subject").value;
  const content = document.getElementById("content").value;
  const statusDiv = document.getElementById("status");

  if (!emails || !subject || !content) {
    statusDiv.innerHTML = "❌ Please fill in all fields!";
    showToast("Please fill in all fields before sending!", "error", 3000);
    return;
  }

  // Check connection before sending
  let isConnected = true;
  if (isElectronApp && window.electron) {
    isConnected = window.electron.connection.check();
  } else {
    isConnected = navigator.onLine;
  }

  if (!isConnected) {
    showToast("⚠️ No internet connection. Please check your connection and try again.", "warning", 5000);
    statusDiv.innerHTML = "❌ No internet connection detected!";
    return;
  }

  statusDiv.innerHTML = "⏳ Sending emails...";

  try {
    const response = await fetch("/send-email", {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify({ emails, subject, content }),
    });

    const result = await response.json();
    statusDiv.innerHTML = result.results
      .map((r) => `${r.email}: ${r.status}`)
      .join("<br>");
    
    // Show success toast
    const successCount = result.results.filter(r => r.status.includes('✅')).length;
    const totalCount = result.results.length;
    showToast(`📧 Sent ${successCount} out of ${totalCount} emails successfully!`, "success", 4000);
    
  } catch (error) {
    statusDiv.innerHTML = `❌ Error: ${error.message}`;
    showToast(`❌ Error sending emails: ${error.message}`, "error", 5000);
  }
});
